; Vector Table Mapped to Address 0 at Reset
; Linker requires __Vectors to be exported
 
          AREA    RESET, DATA, READONLY
          EXPORT  __Vectors
 
__Vectors 
			DCD  0x20001000     ; stack pointer value when stack is empty
			DCD  Reset_Handler  ; reset vector
consta		DCD	 	0x4			; polynomial value a
constb		DCD		0x7			; polynomial value b
constc		DCD		0x8			; polynomial value c
constd		DCD		0x50		; constant threshold value to compare y	
constx		DCD		0x3			; value of variable x 
			ALIGN

; data area that can be modified (readwrite)
			AREA Data, DATA, READWRITE
solutiony	DCD		0x0		 	;just allocating bytes to store result if need be
			ALIGN
; The program
; Linker requires Reset_Handler
 
			AREA    MYCODE, CODE, READONLY
			ENTRY
			EXPORT Reset_Handler
			ALIGN 2
;;;;;Procedure definitions;;;;
			
polynomial	 	PROC
;; r0 contains value of x to be used for ax^2 + bx + c
		STMFD SP!, {R1-R8, LR}		; push all registers used in subroutine to prevent corruption 
		LDR		r1,consta			; load values for polynomial constants a, b, c
		LDR		r2,constb			
		LDR		r3,constc
		; compute x^2	
		MUL		r4,r0,r0
		; compute (a times x^2) + c
		MLA		r4,r1,r4,r3
		; compute	y = (b times x) + (ax^2 + c)		y is stored in r0
		MLA		r0,r2,r0,r4
		; load threshold value d
		LDR		r1,constd
		; compare y > d
		CMP		r0,r1
		; mov d to r0 if y is greater than d else return y in r0
		MOVGT	r0,r1
		LDMFD SP!, {R1-R8, PC}		; pop all  registers and branch back to main program
				ENDP
		
Reset_Handler
;;;;;;;;;;User Code Starts from the next line;;;;;;;;;;;;
;; code to copy data from ROM to RAM on startup i.e. from readonly to readwrite area in memory
			
; any other startup code here
				ALIGN 2
			B		mainprog

mainprog	;;add your code here to 
			LDR			r0,constx		; load value of variable x
			BL			polynomial		; call function to calculate value of polynomial
										; result is returned in reg r0
			MOV			r1,r0,LSL #0x1	; store 2 times r0 into r1
			B		.					; end program
			END